<p:config xmlns:p="http://www.orbeon.com/oxf/pipeline"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:oxf="http://www.orbeon.com/oxf/processors"
    xmlns:xforms="http://www.w3.org/2002/xforms"
    xmlns:l="http://orbeon.org/layout"
    xmlns:xh="http://www.w3.org/1999/xhtml">

    <p:param type="input" name="data" />
    <p:param type="input" name="instance"/>
    <p:param type="input" name="xforms-model"/>

    <!-- Annotate XForms elements and generate XHTML if necessary -->
    <p:choose href="#xforms-model">
        <p:when test="/xforms:model">
            <p:processor name="oxf:xforms-output">
                <p:input name="model" href="#xforms-model"/>
                <p:input name="instance" href="#instance"/>
                <p:input name="data" href="#data"/>
                <p:output name="data" id="annotated-data"/>
            </p:processor>
            <!-- Transform annotated XForms to XHTML -->
            <p:processor name="oxf:xslt">
                <p:input name="config" href="xforms-to-xhtml.xsl"/>
                <p:input name="model" href="#xforms-model"/>
                <p:input name="instance" href="#instance"/>
                <p:input name="data" href="#annotated-data"/>
                <p:output name="data" id="xhtml-data"/>
            </p:processor>
            <!-- Aggregate hidden fields -->
            <p:processor name="oxf:xslt">
                <p:input name="config" href="xforms-hidden.xsl"/>
                <p:input name="data" href="#xhtml-data"/>
                <p:output name="data" id="xformed-data"/>
            </p:processor>
        </p:when>
        <p:otherwise>
            <p:processor name="oxf:identity">
                <p:input name="data" href="#data"/>
                <p:output name="data" id="xformed-data"/>
            </p:processor>
        </p:otherwise>
    </p:choose>
    
    <p:processor name="oxf:scope-generator">
        <p:input name="config">
            <config>
                <key>sf-news</key>
                <scope>application</scope>
            </config>
        </p:input>
        <p:output name="data" id="sf-news"/>
    </p:processor>
    
    <p:processor name="oxf:xslt">
        <p:input name="data" href="#xformed-data"/>
        <p:input name="sf-news" href="#sf-news"/>
        <p:input name="config">
            <xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
                <xsl:import href="oxf:/oxf/xslt/utils/copy.xsl"/>
                <xsl:template match="l:sf-news-feed">
                    <l:box title="Community News">
                        <xsl:for-each select="doc('oxf:sf-news')/rss/channel/item">
                            <l:line>
                                <xsl:value-of select="replace(pubDate, '.... ([0-9]* ... ....) .*', '$1')"/><br/>
                                <xh:a href="{link}">
                                    <xsl:value-of select="title"/>
                                </xh:a>
                            </l:line>
                        </xsl:for-each>
                        <l:line>
                            <div style="margin-right: 1em; text-align: right">
                                <a href="http://sourceforge.net/export/rss2_projnews.php?group_id=116683">
                                    <img border="0" src="/images/xml.png"/>
                                </a>
                            </div>
                        </l:line>
                    </l:box>
                </xsl:template>
            </xsl:stylesheet>
        </p:input>
        <p:output name="data" id="rssed-data"/>
    </p:processor>

    <!-- Get container type -->
    <p:processor name="oxf:request">
        <p:input name="config">
            <config>
                <include>/request/container-type</include>
                <include>/request/path-info</include>
            </config>
        </p:input>
        <p:output name="data" id="request"/>
    </p:processor>

    <p:processor uri="oxf/processor/request-security" xmlns:p="http://www.orbeon.com/oxf/pipeline">
        <p:input name="config">
            <config>
                <role>user</role>
            </config>
        </p:input>
        <p:output name="data" id="request-security"/>
    </p:processor>

    <!-- Apply theme -->
    <p:processor name="oxf:xslt">
        <p:input name="data" href="#rssed-data"/>
        <p:input name="nav" href="/navigation.xml"/>
        <p:input name="request" href="#request"/>
        <p:input name="sec" href="#request-security"/>
        <p:input name="config" href="/orbeon-theme/theme.xsl"/>
        <p:output name="data" id="themed-data"/>
    </p:processor>
    <!-- Rewrite all URLs in HTML and XHTML documents -->
    <p:processor name="oxf:xslt">
        <p:input name="data" href="#themed-data"/>
        <p:input name="config" href="oxf:/oxf/pfc/oxf-rewrite.xsl"/>
        <p:output name="data" id="rewritten-data"/>
    </p:processor>
    <!-- Output regular HTML doctype -->
    <p:processor name="oxf:html-serializer">
        <p:input name="config">
            <config>
                <public-doctype>-//W3C//DTD HTML 4.01 Transitional//EN</public-doctype>
                <version>4.01</version>
                <encoding>utf-8</encoding>
            </config>
        </p:input>
        <p:input name="data" href="#rewritten-data"/>
    </p:processor>

</p:config>
