<p:config xmlns:p="http://www.orbeon.com/oxf/pipeline">
    <p:param type="input" name="instance"/>
    <p:param type="output" name="data"/>

    <p:processor uri="oxf/processor/sql">
        <p:input name="data" href="#instance"/>
        <p:input name="config">
            <sql:config xmlns:sql="http://orbeon.org/oxf/xml/sql">
                <sql:connection>
                    <sql:datasource>db</sql:datasource>
                    <sql:execute>
                        <sql:query>
                             SELECT email FROM users
                               WHERE email =
                            <sql:param type="xs:string" select="/form/email"/>
                        </sql:query>
                        <sql:results>
                            <existing/>
                        </sql:results>
                        <sql:no-results>
                            <ok/>
                        </sql:no-results>
                    </sql:execute>
                </sql:connection>
            </sql:config>
        </p:input>
        <p:output name="data" id="existing"/>
    </p:processor>


    <p:choose href="#existing">
        <p:when test="/ok">
            <p:processor uri="oxf/processor/xslt-2.0">
                <p:input name="data">
                    <dummy/>
                </p:input>
                <p:input name="config">
                    <xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:rand="java:java.util.Random">
                        <xsl:variable name="r" select="rand:new()"/>
                        <xsl:template match="/">
                            <hash>
                                <xsl:for-each select="1 to 32">
                                    <xsl:value-of select="(rand:nextInt($r, 10))"/>
                                </xsl:for-each>
                            </hash>
                        </xsl:template>
                    </xsl:stylesheet>
                </p:input>
                <p:output name="data" id="hash" />
            </p:processor>

            <p:processor uri="oxf/processor/sql">
                <p:input name="data" href="aggregate('root', #hash, #instance)"/>
                <p:input name="config">
                    <sql:config xmlns:sql="http://orbeon.org/oxf/xml/sql">
                        <sql:connection>
                            <sql:datasource>db</sql:datasource>
                            <success/>
                            <sql:execute>
                                <sql:update>
                                      INSERT INTO users
                                         (email,
                                          verified,
                                          firstname,
                                          lastname,
                                          company,
                                          title,
                                          phone,
                                          devTeam,
                                          source,
                                          appServer,
                                          ide,
                                          target,
                                          issue,
                                          date,
                                          password,
                                          interest,
                                          hash)
                                          VALUES (
                                    <sql:param type="xs:string" select="/root/form/email"/>,
                                           0,
                                    <sql:param type="xs:string" select="/root/form/first-name"/>,
                                    <sql:param type="xs:string" select="/root/form/last-name"/>,
                                    <sql:param type="xs:string" select="/root/form/company"/>,
                                    <sql:param type="xs:string" select="/root/form/title"/>,
                                    <sql:param type="xs:string" select="/root/form/work-phone"/>,
                                    <sql:param type="xs:string" select="/root/form/team-size"/>,
                                    <sql:param type="xs:string" select="/root/form/source"/>,
                                    <sql:param type="xs:string" select="/root/form/app-server"/>,
                                    <sql:param type="xs:string" select="/root/form/ide"/>,
                                    <sql:param type="xs:string" select="/root/form/target"/>,
                                    <sql:param type="xs:string" select="/root/form/issue"/>,
                                           curdate(),
                                    <sql:param type="xs:string" select="/root/form/password1"/>,
                                    <sql:param type="xs:string" select="/root/form/interest"/>,
                                    <sql:param type="xs:string" select="/root/hash"/>
                                          );
                                </sql:update>
                            </sql:execute>
                        </sql:connection>
                    </sql:config>
                </p:input>
                <p:output name="data" id="abc"/>
            </p:processor>

            <p:processor uri="oxf/processor/xslt-2.0">
                <p:input name="data" href="aggregate('root', #hash, #instance, #abc)"/>
                <p:input name="config">
                    <xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
                        <xsl:template match="/">
                            <message>
                                <smtp-host>localhost</smtp-host>
                                <from>
                                    <email>oxf-info@orbeon.com</email>
                                    <name>OXF Info</name>
                                </from>
                                <to>
                                    <email><xsl:value-of select="/root/form/email"/></email>
                                    <name><xsl:value-of select="/root/form/first-name"/> <xsl:value-of select="/root/form/last-name"/></name>
                                </to>
                                <subject>OXF - Welcome</subject>
                                <text>
Dear <xsl:value-of select="/root/form/first-name"/>

Thank you for your interest in Orbeon. Click on the URL below to confirm
your My Orbeon registration under the email address <xsl:value-of select="/root/form/email"/>.

http://www.orbeon.com/login/confirm?<xsl:value-of select="/root/hash"/>

After you have confirmed your registration, you will be able to access the
My Orbeon site at http://www.orbeon.com/my/ .

If you have trouble confirming your registration or have questions about your registration,
please email info@orbeon.com.

Best regards,

-The Orbeon Team
                                </text>
                            </message>
                        </xsl:template>
                    </xsl:stylesheet>
                </p:input>
                <p:output name="data" id="email"/>
            </p:processor>

            <p:processor uri="oxf/processor/email" xmlns:p="http://www.orbeon.com/oxf/pipeline">
                <p:input name="data" href="#email"/>
            </p:processor>
            <p:processor uri="oxf/processor/xslt-2.0">
                <p:input name="data" href="#instance"/>
                <p:input name="config">
                    <xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
                        <xsl:template match="/">
                            <message>
                                <smtp-host>localhost</smtp-host>
                                <from>
                                    <email>oxf-info@orbeon.com</email>
                                    <name>OXF Info</name>
                                </from>
                                <to>
                                    <email>oxf-downloads@orbeon.com</email>
                                    <name>OXF Downloads</name>
                                </to>
                                <subject>OXF Registration (<xsl:value-of select="/form/company"/>)</subject>
                                <text>
New OXF User:

Email: <xsl:value-of select="/form/email"/>
First Name: <xsl:value-of select="/form/first-name"/>
Last Name: <xsl:value-of select="/form/last-name"/>
Company: <xsl:value-of select="/form/company"/>
Title: <xsl:value-of select="/form/title"/>
Phone: <xsl:value-of select="/form/work-phone"/>
Team Size: <xsl:value-of select="/form/team-size"/>
Source: <xsl:value-of select="/form/source"/>
App Server: <xsl:value-of select="/form/app-server"/>
IDE: <xsl:value-of select="/form/ide"/>
Target: <xsl:value-of select="/form/target"/>
Issue: <xsl:value-of select="/form/issue"/>
                                </text>
                            </message>
                        </xsl:template>
                    </xsl:stylesheet>
                </p:input>
                <p:output name="data" id="email-download"/>
            </p:processor>

            <p:processor uri="oxf/processor/email" xmlns:p="http://www.orbeon.com/oxf/pipeline">
                <p:input name="data" href="#email-download"/>
            </p:processor>

            <p:processor uri="oxf/processor/identity">
                <p:input name="data" href="#abc"/>
                <p:output name="data" ref="data"/>
            </p:processor>

        </p:when>
        <p:otherwise>
            <p:processor uri="oxf/processor/identity">
                <p:input name="data" href="#existing"/>
                <p:output name="data" ref="data"/>
            </p:processor>

        </p:otherwise>
    </p:choose>


</p:config>